import { ref } from 'vue';
import { useRouter } from 'vue-router';
import { useNuxtApp } from '#app';

export default function useLogin() {
  const $sdk = useNuxtApp().$sdk;
  const router = useRouter();
  const username = ref('');
  const password = ref('');
  const hiddenPassword = ref(false);
  const notify = ref({ type: '', message: '' });
  const errors = ref({ username: '', password: '' });
  const loading = ref(false);

  // Define a configurable redirect path
  const defaultRedirectPath = '/dashboard'; // Replace with your initial default
  const redirectPath = ref(defaultRedirectPath);

  const handleSubmit = async () => {
    // Reset errors
    errors.value = { username: '', password: '' };

    // Validate form fields
    if (!username.value.trim()) {
      errors.value.username = 'Username is required';
      return;
    }
    if (!password.value.trim()) {
      errors.value.password = 'Password is required';
      return;
    }

    // Loading state
    loading.value = true;

    try {
      // Login API call
      const response = await $sdk.authorization.login({
        username: username.value,
        password: password.value,
      });

      // Handle successful login
      if (response) {
        // Use the configured redirect path
        router.push(redirectPath.value);
      } else {
        notify.value = {
          type: 'error',
          message: 'Login failed. Please check your credentials.',
        };
      }
    } catch (error) {
      console.error('Login error:', error);
      notify.value = {
        type: 'error',
        message: 'An unexpected error occurred. Please try again later.',
      };
    } finally {
      loading.value = false;
    }
  };

  const togglePasswordVisibility = () => {
    hiddenPassword.value = !hiddenPassword.value;
  };

  // Allow setting the redirect path from outside the composable
  const setRedirectPath = (newPath: string) => {
    redirectPath.value = newPath;
  };

  return {
    username,
    password,
    hiddenPassword,
    notify,
    errors,
    loading,
    handleSubmit,
    togglePasswordVisibility,
    setRedirectPath,
  };
}
